from xmlrpclib import ServerProxy, Transport, ProtocolError
from base64 import encodestring
from httplib import HTTP


class BasicAuthTransport(Transport):
    """ taken from http://www.zope.org/Members/Amos/XML-RPC """

    def __init__(self, username=None, password=None, verbose=0, timeout=None):
        """ initialize the transports authentication values """
        self.username = username
        self.password = password
        self.verbose = verbose
        self.timeout = timeout

    def request(self, host, handler, request_body, verbose=0):
	""" issue XML-RPC request """
        http = HTTP(host, timeout=self.timeout)
        http.putrequest('POST', handler)
        http.putheader('Host', host)
        http.putheader('User-Agent', self.user_agent)
        http.putheader('Content-Type', 'text/xml')
        http.putheader('Content-Length', str(len(request_body)))

        if self.username is not None and self.password is not None:
	    token = encodestring('%s:%s' % (self.username, self.password))
	    token = token.replace('\n', '')
            http.putheader('Authorization', 'Basic %s' % token)
        http.endheaders()

        if request_body:
            http.send(request_body)

        errcode, errmsg, headers = http.getreply()
        if not errcode == 200:
            raise ProtocolError(host + handler, errcode, errmsg, headers)

        return self.parse_response(http.getfile())


def authServerProxy(uri, login, password, verbose=None, timeout=None, **args):
    """ convenience function to return a server object using basic auth """
    trans = BasicAuthTransport(login, password, verbose, timeout)
    server = ServerProxy(uri, trans, **args)
    return server

